"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createGeneralValidateMiddleware = exports.extractAndValidate = exports.extract = exports.generalFinalValidationCallbacks = exports.generalValidateCallbacks = void 0;
var _1 = require(".");
var validateUtils_1 = __importStar(require("./validateUtils"));
var _a = validateUtils_1.default(createGeneralValidation, createGeneralValidateMiddleware), stringValidate = _a.stringValidate, numberValidate = _a.numberValidate;
exports.generalValidateCallbacks = {
    maxLength: stringValidate.maxLength,
    exactLength: stringValidate.exactLength
};
exports.generalFinalValidationCallbacks = {
    greaterOrEqualTo: numberValidate.greaterOrEqualTo
};
function extract(target, rules) {
    var constructErrorMessage = function (v, s) { return "The data does not comply with the rules: " + v + " " + s; };
    if (target === null || target === undefined || !Object.getPrototypeOf(target).isPrototypeOf(Object))
        throw new Error(constructErrorMessage(target, "should be an object"));
    var out = {};
    for (var requiredKey in rules) {
        var entry = target[requiredKey];
        var entryRequirementsInfo = rules[requiredKey];
        if (entry === undefined) {
            if (entryRequirementsInfo.required)
                throw new Error(constructErrorMessage(entry, "is required and should be present"));
            else
                continue;
        }
        if (typeof entryRequirementsInfo.type === "string") {
            // check if the data is correct
            var dataClass = _1.fieldTypesByTypeName[entryRequirementsInfo.type];
            if (dataClass !== undefined && (entry instanceof dataClass || typeof entry === entryRequirementsInfo.type))
                out[requiredKey] = entry;
            else
                throw new Error(constructErrorMessage(entry, "should be a " + entryRequirementsInfo.type));
        }
        else {
            // if it is a nested array, recurse
            out[requiredKey] = extract(entry, entryRequirementsInfo.type);
        }
    }
    return out;
}
exports.extract = extract;
function extractAndValidate(target, rules) {
    var extractedTarget = extract(target, rules); // make sure that all the needed keys are there
    var finalValidationCalls = [];
    function recursivelyApplySettings(currentTarget, currentRules) {
        for (var currentElementName in currentTarget) {
            var currentElement = currentTarget[currentElementName];
            var currentRule = currentRules[currentElementName];
            // if a flattened object, do it recursively
            if (typeof currentRule.type !== "string") {
                recursivelyApplySettings(currentElement, currentRule.type);
                continue;
            }
            for (var optionName in currentRule) {
                if (optionName === "type" || optionName === "required")
                    continue; // skip if it is not a rule
                var option = currentRule[optionName];
                if (optionName === "enum") {
                    if (!option.includes(currentElement))
                        throw new Error("The enum \"" + currentElementName + "\" with the value \"" + currentElement + "\" must have one of the following values: \"" + option + "\"");
                }
                if (optionName in exports.generalValidateCallbacks) {
                    try {
                        exports.generalValidateCallbacks[optionName](currentElement, option);
                    }
                    catch (e) {
                        throw new Error("The \"" + optionName + "\" constraint with the value \"" + option + "\" was not met by the field \"" + currentElementName + "\" with the value \"" + currentElement + "\" with an error message saying \"" + e.message + "\"");
                    }
                }
                if (optionName in exports.generalFinalValidationCallbacks) {
                    // the first value is the validation callback, the others are the values to check
                    var outVals = [exports.generalFinalValidationCallbacks[optionName], currentElementName];
                    if (Array.isArray(option))
                        outVals.push.apply(outVals, option);
                    else
                        outVals.push(option);
                    finalValidationCalls.push(outVals);
                    continue;
                }
            }
        }
    }
    recursivelyApplySettings(extractedTarget, rules);
    for (var callName in finalValidationCalls) {
        var callOptions = finalValidationCalls[callName];
        var validator = callOptions.shift();
        try {
            validator(callOptions, extractedTarget);
        }
        catch (e) {
            throw new Error("A constraint failed with an error message saying \"" + e.message + "\"");
        }
    }
    return extractedTarget;
}
exports.extractAndValidate = extractAndValidate;
function createGeneralValidation(message, validate) {
    return function (value) {
        var checkSettings = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            checkSettings[_i - 1] = arguments[_i];
        }
        var result = validate.apply(null, __spreadArray([value], checkSettings));
        if (!result)
            throw new Error(message);
        return result;
    };
}
function createGeneralValidateMiddleware(message, automaticResponseToUndefined, callback) {
    return function (fieldsRequiredToValidate, toValidate) {
        if (validateUtils_1.callOverallValidationFunction(fieldsRequiredToValidate, toValidate, automaticResponseToUndefined, callback))
            return true;
        throw new Error(message + "; Fields for validation: " + fieldsRequiredToValidate);
    };
}
exports.createGeneralValidateMiddleware = createGeneralValidateMiddleware;
//# sourceMappingURL=general.js.map