"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from) {
    for (var i = 0, il = from.length, j = to.length; i < il; i++, j++)
        to[j] = from[i];
    return to;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.mongoCreateValidateMiddleware = exports.mongoCreateValidation = exports.constructSchema = exports.mongoFinalValidationCallbacks = exports.mongoValidateCallbacks = exports.mongoValuesToApply = void 0;
var mongoose_1 = require("mongoose");
var validateUtils_1 = __importStar(require("./validateUtils"));
var utils_1 = require("./utils");
var _a = validateUtils_1.default(mongoCreateValidation, mongoCreateValidateMiddleware), stringValidate = _a.stringValidate, numberValidate = _a.numberValidate;
exports.mongoValuesToApply = ["default", "enum", "unique"];
exports.mongoValidateCallbacks = {
    maxLength: stringValidate.maxLength,
    exactLength: stringValidate.exactLength
};
exports.mongoFinalValidationCallbacks = {
    greaterOrEqualTo: numberValidate.greaterOrEqualTo
};
function constructSchema(inputFields) {
    var flattened = utils_1.flatten(inputFields);
    function recursivelyApplySettings(fields) {
        var schemaSettings = {};
        var finalValidationCallbacksForThisSchema = [];
        for (var paramName in fields) {
            var paramOptions = fields[paramName];
            // if a flattened object, do it recursively
            if (typeof paramOptions.type !== "string") {
                schemaSettings[paramName] = {
                    type: recursivelyApplySettings(paramOptions.type),
                    required: paramOptions.required,
                };
                continue;
            }
            var currentEntry = { type: utils_1.fieldTypesByTypeName[paramOptions.type], required: paramOptions.required };
            for (var paramOptionName in paramOptions) {
                if (paramOptionName === "type" || paramOptionName === "required")
                    continue;
                var paramOption = paramOptions[paramOptionName];
                if (paramOptionName in exports.mongoValidateCallbacks) {
                    currentEntry["validate"] = exports.mongoValidateCallbacks[paramOptionName](paramOption);
                    continue;
                }
                if (exports.mongoValuesToApply.includes(paramOptionName)) {
                    currentEntry[paramOptionName] = paramOption;
                    continue;
                }
                if (paramOptionName in exports.mongoFinalValidationCallbacks) {
                    // the first value is the validation callback, the others are the values to check
                    var outVals = [exports.mongoFinalValidationCallbacks[paramOptionName], paramName];
                    if (Array.isArray(paramOption))
                        outVals.push.apply(outVals, paramOption);
                    else
                        outVals.push(paramOption);
                    finalValidationCallbacksForThisSchema.push(outVals);
                    continue;
                }
            }
            schemaSettings[paramName] = currentEntry;
        }
        var out = new mongoose_1.Schema(schemaSettings);
        finalValidationCallbacksForThisSchema.forEach(function (settings) {
            var validator = settings.shift();
            out.pre("validate", validator(settings));
        });
        return out;
    }
    return recursivelyApplySettings(flattened);
}
exports.constructSchema = constructSchema;
// "current" means that those values are no longer general
function mongoCreateValidation(message, validate) {
    return function () {
        var checkSettings = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            checkSettings[_i] = arguments[_i];
        }
        return {
            message: message,
            validator: function (value) { return validate.apply(null, __spreadArray([value], checkSettings)); }
        };
    };
}
exports.mongoCreateValidation = mongoCreateValidation;
function mongoCreateValidateMiddleware(message, automaticResponseToUndefined, callback) {
    return function (fieldsRequiredToValidate) {
        var out = function (next) {
            if (validateUtils_1.callOverallValidationFunction(fieldsRequiredToValidate, this, automaticResponseToUndefined, callback))
                return next();
            throw new Error(message + "; Fields for validation: " + fieldsRequiredToValidate);
        };
        return out;
    };
}
exports.mongoCreateValidateMiddleware = mongoCreateValidateMiddleware;
//# sourceMappingURL=mongo.js.map