"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.fieldTypesByTypeName = exports.deleteEmptyObjectsAndUndefined = exports.flatten = exports.deepAssign = void 0;
//NOTE: this function is NOT SAFE and therefore should never be used with user input
function deepAssign(target, src) {
    if (src === undefined)
        return target; // return target if there is nothing in src to override target with
    var isRecursible = function (val) { return typeof val === "object" && val !== null; };
    if (!isRecursible(target) || !isRecursible(src))
        return src; // if we can not recurse anymore, then choose src
    var keys = new Set(Object.keys(target));
    Object.keys(src).forEach(function (k) { return keys.add(k); }); // a set allows us not to bother about duplicates
    var out = (Array.isArray(target)) ? [] : {}; // the original Object.assign seems to select the type based on the target
    keys.forEach(function (k) {
        out[k] = deepAssign(target[k], src[k]);
    });
    return out;
}
exports.deepAssign = deepAssign;
//NOTE: this function is NOT SAFE and therefore should never be used with user input
function flatten(fields) {
    var out = {};
    for (var typeContainerKey in fields) {
        var typeContainer = fields[typeContainerKey];
        var isRequired = typeContainerKey === "required";
        for (var groupName in typeContainer) {
            // if an object, recurse
            var fieldGroup = typeContainer[groupName];
            for (var entryName in fieldGroup) {
                var additionalData = fieldGroup[entryName];
                // recurse if needed
                if (groupName === "object") {
                    out[entryName] = {
                        type: flatten(additionalData),
                        "required": isRequired,
                    };
                    continue;
                }
                // else, copy over all the needed values
                out[entryName] = {
                    type: groupName,
                    required: isRequired,
                };
                for (var additionalDataName in additionalData) {
                    out[entryName][additionalDataName] = additionalData[additionalDataName];
                }
            }
        }
    }
    return out;
}
exports.flatten = flatten;
// NOTE: also deletes undefined
function deleteEmptyObjectsAndUndefined(target) {
    // if not a plain object, skip
    if (target === null || target === undefined || !Object.getPrototypeOf(target).isPrototypeOf(Object))
        return target;
    if (Object.keys(target).length === 0)
        return undefined;
    var out = {};
    for (var objKey in target) {
        // if an empty object, do not add
        var newVal = deleteEmptyObjectsAndUndefined(target[objKey]);
        if (newVal !== undefined)
            out[objKey] = newVal;
    }
    return out;
}
exports.deleteEmptyObjectsAndUndefined = deleteEmptyObjectsAndUndefined;
exports.fieldTypesByTypeName = {
    string: String,
    number: Number,
    boolean: Boolean,
    date: Date,
    object: Object,
};
//# sourceMappingURL=utils.js.map